@@ -27,20 +27,25 @@
 
 #include "gst.h"
 #include "gstqueue.h"
-#ifndef GST_DISABLE_TYPE_FIND
-#include "gsttypefind.h"
-#endif
+#ifndef GST_DISABLE_REGISTRY
 #include "registries/gstxmlregistry.h"
+#endif /* GST_DISABLE_REGISTRY */
+#include "gstregistrypool.h"
+
+#define GST_CAT_DEFAULT GST_CAT_GST_INIT
 
 #define MAX_PATH_SPLIT	16
 #define GST_PLUGIN_SEPARATOR ","
 
 gchar *_gst_progname;
 
+#ifndef GST_DISABLE_REGISTRY
 gboolean _gst_registry_auto_load = TRUE;
 static GstRegistry *_global_registry;
 static GstRegistry *_user_registry;
 static gboolean _gst_registry_fixed = FALSE;
+#endif
+
 static gboolean _gst_use_threads = TRUE;
 
 static gboolean _gst_enable_cpu_opt = TRUE;
@@ -50,6 +55,9 @@ static gboolean gst_initialized = FALSE;
 static gboolean _gst_initialization_failure = FALSE;
 extern gint _gst_trace_on;
 
+/* set to TRUE when segfaults need to be left as is */
+gboolean _gst_disable_segtrap = FALSE;
+
 extern GThreadFunctions gst_thread_dummy_functions;
 
 
@@ -80,16 +88,19 @@ debug_log_handler (const gchar *log_domain,
 enum {
   ARG_VERSION=1,
   ARG_FATAL_WARNINGS,
-  ARG_INFO_MASK,
-  ARG_DEBUG_MASK,
-  ARG_MASK,
-  ARG_MASK_HELP,
+#ifndef GST_DISABLE_GST_DEBUG 
+  ARG_DEBUG_LEVEL,
+  ARG_DEBUG,
+  ARG_DEBUG_DISABLE,
+  ARG_DEBUG_NO_COLOR,
+  ARG_DEBUG_HELP,
+#endif
   ARG_DISABLE_CPU_OPT,
   ARG_PLUGIN_SPEW,
   ARG_PLUGIN_PATH,
   ARG_PLUGIN_LOAD,
+  ARG_SEGTRAP_DISABLE,
   ARG_SCHEDULER,
-  ARG_NOTHREADS,
   ARG_REGISTRY
 };
 
@@ -97,24 +108,27 @@ enum {
 #define NUL '\0'
 #endif
 
-/* default scheduler, 'opt', can be changed in
- * gstscheduler.c in function gst_scheduler_factory_class_init
+/* default scheduler, can be changed in gstscheduler.h with
+ * the GST_SCHEDULER_DEFAULT_NAME define.
  */
-static const struct poptOption options[] = {
-  {NULL, NUL, POPT_ARG_CALLBACK|POPT_CBFLAG_PRE|POPT_CBFLAG_POST, &init_popt_callback, 0, NULL, NULL},
-  {"gst-version",        NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_VERSION,        "Print the GStreamer version", NULL},
-  {"gst-fatal-warnings", NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_FATAL_WARNINGS, "Make all warnings fatal", NULL},
-  {"gst-info-mask",      NUL, POPT_ARG_INT|POPT_ARGFLAG_STRIP,    NULL, ARG_INFO_MASK,      "info bitmask", "MASK"},
-  {"gst-debug-mask",     NUL, POPT_ARG_INT|POPT_ARGFLAG_STRIP,    NULL, ARG_DEBUG_MASK,     "debugging bitmask", "MASK"},
-  {"gst-mask",           NUL, POPT_ARG_INT|POPT_ARGFLAG_STRIP,    NULL, ARG_MASK,           "bitmask for both info and debugging", "MASK"},
-  {"gst-mask-help",      NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_MASK_HELP,      "how to set the level of diagnostic output (-mask values)", NULL},
-  {"gst-disable-cpu-opt",NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,	  NULL, ARG_DISABLE_CPU_OPT,"Disable accelerated CPU instructions", NULL},
-  {"gst-plugin-spew",    NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_PLUGIN_SPEW,    "enable verbose plugin loading diagnostics", NULL},
-  {"gst-plugin-path",    NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_PLUGIN_PATH,    "'" G_SEARCHPATH_SEPARATOR_S "'--separated path list for loading plugins", "PATHS"},
-  {"gst-plugin-load",    NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_PLUGIN_LOAD,    "comma-separated list of plugins to preload in addition to the list stored in env variable GST_PLUGIN_PATH", "PLUGINS"},
-  {"gst-scheduler",      NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_SCHEDULER,      "scheduler to use ('opt' is the default)", "SCHEDULER"},
-  {"gst-nothreads",      NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_NOTHREADS,      "use NOPs for all threading and locking operations", NULL},
-  {"gst-registry",       NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_REGISTRY,       "registry to use" , "REGISTRY"},
+static const struct poptOption gstreamer_options[] = {
+  {NULL, NUL, POPT_ARG_CALLBACK|POPT_CBFLAG_PRE|POPT_CBFLAG_POST, (void *) &init_popt_callback, 0, NULL, NULL},
+  {"gst-version",        NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_VERSION,        N_("Print the GStreamer version"), NULL},
+  {"gst-fatal-warnings", NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_FATAL_WARNINGS, N_("Make all warnings fatal"), NULL},
+#ifndef GST_DISABLE_GST_DEBUG 
+  {"gst-debug-level",    NUL, POPT_ARG_INT|POPT_ARGFLAG_STRIP,    NULL, ARG_DEBUG_LEVEL,  N_("default debug level from 1 (only error) to 5 (anything) or 0 for no output"), "LEVEL"},
+  {"gst-debug",          NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_DEBUG,          N_("colon-seperated list of category_name=level pairs to set specific levels for the individual categories.\nExample:GST_AUTOPLUG=5:GST_ELEMENT_*=3"), "CATEGORIES"},
+  {"gst-debug-no-color", NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_DEBUG_NO_COLOR, N_("disable color debugging output"), NULL},
+  {"gst-disable-debug",  NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_DEBUG_DISABLE,  N_("disable debugging")},
+  {"gst-debug-help",     NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_DEBUG_HELP,     N_("print available debug categories and exit"), NULL},
+#endif
+  {"gst-disable-cpu-opt",NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,	  NULL, ARG_DISABLE_CPU_OPT,N_("Disable accelerated CPU instructions"), NULL},
+  {"gst-plugin-spew",    NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_PLUGIN_SPEW,    N_("enable verbose plugin loading diagnostics"), NULL},
+  {"gst-plugin-path",    NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_PLUGIN_PATH,    N_("'" G_SEARCHPATH_SEPARATOR_S "'--separated path list for loading plugins"), "PATHS"},
+  {"gst-plugin-load",    NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_PLUGIN_LOAD,    N_("comma-separated list of plugins to preload in addition to the list stored in env variable GST_PLUGIN_PATH"), "PLUGINS"},
+  {"gst-disable-segtrap",NUL, POPT_ARG_NONE|POPT_ARGFLAG_STRIP,   NULL, ARG_SEGTRAP_DISABLE,N_("disable trapping of segmentation faults during plugin loading"), NULL},
+  {"gst-scheduler",      NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_SCHEDULER,      N_("scheduler to use ('"GST_SCHEDULER_DEFAULT_NAME"' is the default)"), "SCHEDULER"},
+  {"gst-registry",       NUL, POPT_ARG_STRING|POPT_ARGFLAG_STRIP, NULL, ARG_REGISTRY,       N_("registry to use") , "REGISTRY"},
   POPT_TABLEEND
 };
 
@@ -132,7 +146,7 @@ static const struct poptOption options[] = {
 const struct poptOption *
 gst_init_get_popt_table (void)
 {
-  return options;
+  return gstreamer_options;
 }
 
 /**
@@ -152,7 +166,7 @@ gst_init_get_popt_table (void)
 gboolean
 gst_init_check (int *argc, char **argv[])
 {
-  return gst_init_with_popt_table (argc, argv, NULL);
+  return gst_init_check_with_popt_table (argc, argv, NULL);
 }
 
 /**
@@ -170,10 +184,7 @@ gst_init_check (int *argc, char **argv[])
 void
 gst_init (int *argc, char **argv[])
 {
-  if (!gst_init_with_popt_table (argc, argv, NULL)) {
-    g_print ("Could not initialize GStreamer !\n");
-    exit (1);
-  }
+  gst_init_with_popt_table (argc, argv, NULL);
 }
 
 /**
@@ -186,72 +197,53 @@ gst_init (int *argc, char **argv[])
  * setting up internal path lists,
  * registering built-in elements, and loading standard plugins.
  *
- * Returns: TRUE when the initialization succeeded.
+ * This function will terminate your program if it was unable to initialize
+ * GStreamer for some reason.  If you want your program to fall back,
+ * use gst_init_check_with_popt_table() instead.
+ */
+void
+gst_init_with_popt_table (int *argc, char **argv[],
+	                  const struct poptOption *popt_options)
+{
+  if (!gst_init_check_with_popt_table (argc, argv, popt_options)) {
+    g_print ("Could not initialize GStreamer !\n");
+    exit (1);
+  }
+}
+/**
+ * gst_init_check_with_popt_table:
+ * @argc: pointer to application's argc
+ * @argv: pointer to application's argv
+ * @popt_options: pointer to a popt table to append
+ *
+ * Initializes the GStreamer library, parsing the options,
+ * setting up internal path lists,
+ * registering built-in elements, and loading standard plugins.
+ *
+ * Returns: TRUE if GStreamer coul be initialized
  */
 gboolean
-gst_init_with_popt_table (int *argc, char **argv[],
-		          const struct poptOption *popt_options)
+gst_init_check_with_popt_table (int *argc, char **argv[],
+		                const struct poptOption *popt_options)
 {
   poptContext context;
-  gint nextopt, i, j, nstrip;
-  gchar **temp;
-  const struct poptOption *options;
+  gint nextopt;
+  struct poptOption *options;
   struct poptOption options_with[] = {
-     POPT_TABLEEND,
-     POPT_TABLEEND,
-     POPT_TABLEEND,
-     POPT_TABLEEND
+    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, poptHelpOptions, 				 0, "Help options:", NULL},
+    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (struct poptOption *) gstreamer_options,	 0, "GStreamer options:", NULL},
+    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (struct poptOption *) popt_options, 		 0, "Application options:", NULL},
+    POPT_TABLEEND
   };
   struct poptOption options_without[] = {
-     POPT_TABLEEND,
-     POPT_TABLEEND,
-     POPT_TABLEEND
+    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, poptHelpOptions, 				 0, "Help options:", NULL},
+    {NULL, NUL, POPT_ARG_INCLUDE_TABLE, (struct poptOption *) gstreamer_options,	 0, "GStreamer options:", NULL},
+    POPT_TABLEEND
   };
 
-  /* This used to be done by struct initialization, but most
-   * compilers don't like calling functions in a struct
-   * initialization.  (It's a GCC extension.)  FIXME: This should
-   * be reworked to look better. */
-  options_with[0].longName   = NULL;
-  options_with[0].shortName  = NUL;
-  options_with[0].argInfo    = POPT_ARG_INCLUDE_TABLE;
-  options_with[0].arg        = poptHelpOptions;
-  options_with[0].val        = 0;
-  options_with[0].descrip    = "Help options:";
-  options_with[0].argDescrip = NULL;
-  options_with[1].longName   = NULL;
-  options_with[1].shortName  = NUL;
-  options_with[1].argInfo    = POPT_ARG_INCLUDE_TABLE;
-  options_with[1].arg        = (struct poptOption *) gst_init_get_popt_table();
-  options_with[1].val        = 0;
-  options_with[1].descrip    = "GStreamer options:";
-  options_with[1].argDescrip = NULL;
-  options_with[2].longName   = NULL;
-  options_with[2].shortName  = NUL;
-  options_with[2].argInfo    = POPT_ARG_INCLUDE_TABLE;
-  options_with[2].arg        = (struct poptOption *) popt_options;
-  options_with[2].val        = 0;
-  options_with[2].descrip    = "Application options:";
-  options_with[2].argDescrip = NULL;
-
-  options_without[0].longName   = NULL;
-  options_without[0].shortName  = NUL;
-  options_without[0].argInfo    = POPT_ARG_INCLUDE_TABLE;
-  options_without[0].arg        = poptHelpOptions;
-  options_without[0].val        = 0;
-  options_without[0].descrip    = "Help options:";
-  options_without[0].argDescrip = NULL;
-  options_without[1].longName   = NULL;
-  options_without[1].shortName  = NUL;
-  options_without[1].argInfo    = POPT_ARG_INCLUDE_TABLE;
-  options_without[1].arg        = (struct poptOption *) gst_init_get_popt_table();
-  options_without[1].val        = 0;
-  options_without[1].descrip    = "GStreamer options:";
-  options_without[1].argDescrip = NULL;
-
   if (gst_initialized)
   {
-    GST_DEBUG (GST_CAT_GST_INIT, "already initialized gst\n");
+    GST_DEBUG ("already initialized gst");
     return TRUE;
   }
 
@@ -289,34 +281,24 @@ gst_init_with_popt_table (int *argc, char **argv[],
     poptFreeContext (context);
     return FALSE;
   }
+
+  *argc = poptStrippedArgv (context, *argc, *argv);
+  
   poptFreeContext (context);
 
-  /* let's do this once there are 1.6.3 popt debs out
-     *argc = poptStrippedArgv (context, *argc, *argv); */
-  
-  /* until then we'll do a very basic arg permutation */
-  temp = *argv + 1;
-  i = 1;
-  nstrip = 0;
-  g_assert (*argc > 0);
-  while (i++ < *argc && *temp[0]=='-') {
-    for (j = 1; j < *argc - 1; j++)
-      (*argv)[j] = (*argv)[j+1];
-    (*argv)[*argc-1] = *temp;
-    nstrip++;
-  }
-  *argc -= nstrip;
   return TRUE;
 }
 
+#ifndef GST_DISABLE_REGISTRY
 static void 
 add_path_func (gpointer data, gpointer user_data)
 {
   GstRegistry *registry = GST_REGISTRY (user_data);
   
-  GST_INFO (GST_CAT_GST_INIT, "Adding plugin path: \"%s\"", (gchar *)data);
+  GST_INFO ("Adding plugin path: \"%s\"", (gchar *) data);
   gst_registry_add_path (registry, (gchar *)data);
 }
+#endif
 
 static void 
 prepare_for_load_plugin_func (gpointer data, gpointer user_data)
@@ -324,32 +306,55 @@ prepare_for_load_plugin_func (gpointer data, gpointer user_data)
   preload_plugins = g_slist_prepend (preload_plugins, data);
 }
 
+static void
+parse_debug_list (const gchar *list)
+{
+  gchar **split;
+  gchar **walk;
+
+  g_return_if_fail (list != NULL);
+
+  walk = split = g_strsplit (list, ":", 0);
+
+  while (walk[0]) {
+    gchar **values = g_strsplit ( walk[0], "=", 2);
+    if (values[0] && values[1]) {
+      gint level = 0;
+      g_strstrip (values[0]);
+      g_strstrip (values[1]);
+      level = strtol (values[1], NULL, 0);
+      if (level >= 0 && level < GST_LEVEL_COUNT) {
+        GST_DEBUG ("setting debugging to level %d for name \"%s\"",
+                       level, values[0]);
+        gst_debug_set_threshold_for_name (values[0], level);
+      }
+    }
+    g_strfreev (values);
+    walk++;
+  }
+  g_strfreev (split);
+}
 static void 
 load_plugin_func (gpointer data, gpointer user_data)
 {
-  gboolean ret;
-  //ret = gst_plugin_load ((gchar *)data);
-  ret = FALSE;
+  GstPlugin *plugin;
+  const gchar *filename;
 
-  if (ret)
-    GST_INFO (GST_CAT_GST_INIT, "Loaded plugin: \"%s\"", (gchar *)data);
-  else
-    GST_INFO (GST_CAT_GST_INIT, "Failed to load plugin: \"%s\"", (gchar *)data);
+  filename = (const gchar *) data;
+
+  plugin = gst_plugin_load_file (filename, NULL);
+
+  if (plugin) {
+    GST_INFO ("Loaded plugin: \"%s\"", filename);
+
+    gst_registry_pool_add_plugin (plugin);
+  } else {
+    GST_WARNING ("Failed to load plugin: \"%s\"", filename);
+  }
 
   g_free (data);
 }
 
-static void 
-parse_number (const gchar *number, guint32 *val)
-{
-  /* handle either 0xHEX or dec */
-  if (*(number+1) == 'x') {
-    sscanf (number+2, "%08x", val);
-  } else {
-    sscanf (number, "%d", val);
-  }
-}
-
 static void
 split_and_iterate (const gchar *stringlist, gchar *separator, GFunc iterator, gpointer user_data) 
 {
@@ -378,60 +383,82 @@ split_and_iterate (const gchar *stringlist, gchar *separator, GFunc iterator, gp
 static gboolean
 init_pre (void)
 {
-  const gchar *homedir;
-  gchar *user_reg;
 
   g_type_init ();
 
-  _global_registry = gst_xml_registry_new ("global_registry", GLOBAL_REGISTRY_FILE);
+  if (g_thread_supported ()) {
+    /* somebody already initialized threading */
+    _gst_use_threads = TRUE;
+  } else {
+    if (_gst_use_threads)
+      g_thread_init (NULL);
+    else
+      g_thread_init (&gst_thread_dummy_functions);
+  }
+  /* we need threading to be enabled right here */
+  _gst_debug_init();
+
+#ifdef ENABLE_NLS
+  bindtextdomain (GETTEXT_PACKAGE, GST_LOCALEDIR);
+  bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+#endif /* ENABLE_NLS */
+
+#ifndef GST_DISABLE_REGISTRY
+  {
+    const gchar *debug_list;
+  
+    debug_list = g_getenv ("GST_DEBUG");
+    if (debug_list) {
+      parse_debug_list (debug_list);
+    }
+  }
+#endif
+#ifndef GST_DISABLE_REGISTRY
+  {
+    gchar *user_reg;
+    const gchar *homedir;
+
+    _global_registry = gst_xml_registry_new ("global_registry", GLOBAL_REGISTRY_FILE);
 
 #ifdef PLUGINS_USE_BUILDDIR
-  /* location libgstelements.so */
-  gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/libs/gst");
-  gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/gst/elements");
-  gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/gst/types");
-  gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/gst/autoplug");
-  gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/gst/schedulers");
-  gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/gst/indexers");
+    /* location libgstelements.so */
+    gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/libs/gst");
+    gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/gst/elements");
+    gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/gst/types");
+    gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/gst/autoplug");
+    gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/gst/schedulers");
+    gst_registry_add_path (_global_registry, PLUGINS_BUILDDIR "/gst/indexers");
 #else
-  /* add the main (installed) library path */
-  gst_registry_add_path (_global_registry, PLUGINS_DIR);
+    /* add the main (installed) library path */
+    gst_registry_add_path (_global_registry, PLUGINS_DIR);
 #endif /* PLUGINS_USE_BUILDDIR */
 
-  if (g_getenv ("GST_REGISTRY"))
-  {
-    user_reg = g_strdup (g_getenv ("GST_REGISTRY"));
-  }
-  else
-  {
-    homedir = g_get_home_dir ();
-    user_reg = g_strjoin ("/", homedir, LOCAL_REGISTRY_FILE, NULL);
-  }
-  _user_registry = gst_xml_registry_new ("user_registry", user_reg);
+    if (g_getenv ("GST_REGISTRY"))
+    {
+      user_reg = g_strdup (g_getenv ("GST_REGISTRY"));
+    }
+    else
+    {
+      homedir = g_get_home_dir ();
+      user_reg = g_strjoin ("/", homedir, LOCAL_REGISTRY_FILE, NULL);
+    }
+    _user_registry = gst_xml_registry_new ("user_registry", user_reg);
 
-  g_free (user_reg);
+    g_free (user_reg);
+  }
+#endif /* GST_DISABLE_REGISTRY */
 
   return TRUE;
 }
 
 static gboolean
-gst_register_core_elements (GModule *module, GstPlugin *plugin)
+gst_register_core_elements (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
   /* register some standard builtin types */
-  factory = gst_element_factory_new ("bin", gst_bin_get_type (), &gst_bin_details);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-  factory = gst_element_factory_new ("pipeline", gst_pipeline_get_type (), &gst_pipeline_details);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-  factory = gst_element_factory_new ("thread", gst_thread_get_type (), &gst_thread_details);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-  factory = gst_element_factory_new ("queue", gst_queue_get_type (), &gst_queue_details);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-#ifndef GST_DISABLE_TYPE_FIND
-  factory = gst_element_factory_new ("typefind", gst_type_find_get_type (), &gst_type_find_details);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-#endif
+  g_assert (gst_element_register (plugin, "bin", GST_RANK_PRIMARY, GST_TYPE_BIN));
+  g_assert (gst_element_register (plugin, "pipeline", GST_RANK_PRIMARY, GST_TYPE_PIPELINE));
+  g_assert (gst_element_register (plugin, "thread", GST_RANK_PRIMARY, GST_TYPE_THREAD));
+  g_assert (gst_element_register (plugin, "queue", GST_RANK_PRIMARY, GST_TYPE_QUEUE));
 
   return TRUE;
 }
@@ -439,8 +466,16 @@ gst_register_core_elements (GModule *module, GstPlugin *plugin)
 static GstPluginDesc plugin_desc = {
   GST_VERSION_MAJOR,           
   GST_VERSION_MINOR,  
-  "gst_core_plugins",
-  gst_register_core_elements
+  "gst_core_elements",
+  "core elements of the GStreamer library",
+  gst_register_core_elements,
+  NULL,
+  VERSION,
+  GST_LICENSE,
+  GST_PACKAGE,
+  GST_ORIGIN,
+
+  GST_PADDING_INIT
 };                         
 
 /*
@@ -461,21 +496,13 @@ init_post (void)
   const gchar *plugin_path;
 #ifndef GST_DISABLE_TRACE
   GstTrace *gst_trace;
-#endif
+#endif /* GST_DISABLE_TRACE */
 
-  if (!g_thread_supported ()) {
-    if (_gst_use_threads)
-      g_thread_init (NULL);
-    else
-      g_thread_init (&gst_thread_dummy_functions);
-  }
-  
   llf = G_LOG_LEVEL_CRITICAL | G_LOG_LEVEL_ERROR | G_LOG_FLAG_FATAL;
   g_log_set_handler (g_log_domain_gstreamer, llf, debug_log_handler, NULL);
   
-  GST_INFO (GST_CAT_GST_INIT, 
-            "Initializing GStreamer Core Library version %s %s",
-            GST_VERSION, _gst_use_threads?"":"(no threads)");
+  GST_INFO ("Initializing GStreamer Core Library version %s %s",
+            VERSION, _gst_use_threads ? "" : "(no threads)");
   
   _gst_format_initialize ();
   _gst_query_type_initialize ();
@@ -485,29 +512,35 @@ init_post (void)
   gst_ghost_pad_get_type ();
   gst_element_factory_get_type ();
   gst_element_get_type ();
-  gst_type_factory_get_type ();
   gst_scheduler_factory_get_type ();
+  gst_type_find_factory_get_type ();
   gst_bin_get_type ();
-#ifndef GST_DISABLE_AUTOPLUG
-  gst_autoplug_factory_get_type ();
-#endif
+#ifndef GST_DISABLE_INDEX
   gst_index_factory_get_type ();
+#endif /* GST_DISABLE_INDEX */
+#ifndef GST_DISABLE_URI
   gst_uri_handler_get_type ();
-
+#endif /* GST_DISABLE_URI */
 
   plugin_path = g_getenv ("GST_PLUGIN_PATH");
+#ifndef GST_DISABLE_REGISTRY
   split_and_iterate (plugin_path, G_SEARCHPATH_SEPARATOR_S, add_path_func, _user_registry);
+#endif /* GST_DISABLE_REGISTRY */
 
   /* register core plugins */
   _gst_plugin_register_static (&plugin_desc);
  
   _gst_cpu_initialize (_gst_enable_cpu_opt);
+  _gst_structure_initialize ();
+  _gst_value_initialize ();
   _gst_props_initialize ();
   _gst_caps_initialize ();
   _gst_plugin_initialize ();
   _gst_event_initialize ();
   _gst_buffer_initialize ();
+  _gst_tag_initialize ();
 
+#ifndef GST_DISABLE_REGISTRY
   if (!_gst_registry_fixed) {
     /* don't override command-line options */
     if (g_getenv ("GST_REGISTRY")) {
@@ -526,6 +559,7 @@ init_post (void)
   if (_gst_registry_auto_load) {
     gst_registry_pool_load_all ();
   }
+#endif /* GST_DISABLE_REGISTRY */
 
   /* if we need to preload plugins */
   if (preload_plugins) {
@@ -548,38 +582,76 @@ init_post (void)
   return TRUE;
 }
 
-static void
-gst_mask_help (void) 
+#ifndef GST_DISABLE_GST_DEBUG
+static gint
+sort_by_category_name (gconstpointer a, gconstpointer b)
 {
-  guint i;
+  return strcmp (gst_debug_category_get_name ((GstDebugCategory *) a),
+		 gst_debug_category_get_name ((GstDebugCategory *) b));
+}
+static void
+gst_debug_help (void) 
+{
+  GSList *list, *walk;
+  GList *list2, *walk2;
   
-  g_print ("\n  Mask (to be OR'ed)   info/debug         FLAGS   \n");
-  g_print ("--------------------------------------------------------\n");
+  if (!init_post ())
+    exit (1);
   
-  for (i = 0; i<GST_CAT_MAX_CATEGORY; i++) {
-    if (gst_get_category_name(i)) {
-
-#if GST_DEBUG_COLOR
-      g_print ("   0x%08x     %s%s     \033[%sm%s\033[00m\n", 1<<i, 
-               (gst_info_get_categories() & (1<<i)?"(enabled)":"         "),
-               (gst_debug_get_categories() & (1<<i)?"/(enabled)":"/         "),
-               _gst_category_colors[i], gst_get_category_name (i));
-#else
-      g_print ("   0x%08x     %s%s     %s\n", 1<<i, 
-               (gst_info_get_categories() & (1<<i)?"(enabled)":"         "),
-               (gst_debug_get_categories() & (1<<i)?"/(enabled)":"/         "),
-               gst_get_category_name (i));
-#endif
+  walk2 = list2 = gst_registry_pool_plugin_list ();
+  while (walk2) {
+    GstPlugin *plugin = GST_PLUGIN (walk2->data);
+    walk2 = g_list_next (walk2);
+    
+    if (!gst_plugin_is_loaded (plugin)) {
+#ifndef GST_DISABLE_REGISTRY 
+      if (GST_IS_REGISTRY (plugin->manager)) {
+	GST_CAT_LOG (GST_CAT_PLUGIN_LOADING, "loading plugin %s", plugin->desc.name);
+	if (gst_registry_load_plugin (GST_REGISTRY (plugin->manager), plugin) != GST_REGISTRY_OK)
+	  GST_CAT_WARNING (GST_CAT_PLUGIN_LOADING, "loading plugin %s failed", plugin->desc.name);
+      }
+#endif /* GST_DISABLE_REGISTRY */
     }
   }
-}
+  g_list_free (list2);
+      
+  list = gst_debug_get_all_categories ();
+  walk = list = g_slist_sort (list, sort_by_category_name);
   
+  g_print ("\n");
+  g_print ("name                  level    description\n");
+  g_print ("---------------------+--------+--------------------------------\n");
+
+  while (walk) {
+    GstDebugCategory *cat = (GstDebugCategory *) walk->data;
+  
+    if (gst_debug_is_colored ()) {
+      gchar *color = gst_debug_construct_term_color (cat->color);
+      g_print ("%s%-20s\033[00m  %1d %s  %s%s\033[00m\n", 
+               color,
+    	       gst_debug_category_get_name (cat),
+               gst_debug_category_get_threshold (cat),
+               gst_debug_level_get_name (gst_debug_category_get_threshold (cat)), 
+  	       color,
+               gst_debug_category_get_description (cat));
+      g_free (color);
+    } else {
+      g_print ("%-20s  %1d %s  %s\n", gst_debug_category_get_name (cat), 
+               gst_debug_category_get_threshold (cat),
+               gst_debug_level_get_name (gst_debug_category_get_threshold (cat)), 
+  	       gst_debug_category_get_description (cat));
+    }
+    walk = g_slist_next (walk);
+  }   
+  g_slist_free (list);
+  g_print ("\n");
+}
+#endif  
 
 static void
 init_popt_callback (poptContext context, enum poptCallbackReason reason,
                     const struct poptOption *option, const char *arg, void *data) 
 {
-  gint val = 0;
   GLogLevelFlags fatal_mask;
 
   if (gst_initialized)
@@ -598,42 +670,52 @@ init_popt_callback (poptContext context, enum poptCallbackReason reason,
       fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
       g_log_set_always_fatal (fatal_mask);
       break;
-    case ARG_INFO_MASK:
-      parse_number (arg, &val);
-      gst_info_set_categories (val);
+#ifndef GST_DISABLE_GST_DEBUG 
+    case ARG_DEBUG_LEVEL: {
+      gint tmp = 0;
+      tmp = strtol (arg, NULL, 0);
+      if (tmp >= 0 && tmp < GST_LEVEL_COUNT) {
+        gst_debug_set_default_threshold (tmp);
+      }
       break;
-    case ARG_DEBUG_MASK:
-      parse_number (arg, &val);
-      gst_debug_set_categories (val);
+    }
+    case ARG_DEBUG:
+      parse_debug_list (arg);
       break;
-    case ARG_MASK:
-      parse_number (arg, &val);
-      gst_debug_set_categories (val);
-      gst_info_set_categories (val);
+    case ARG_DEBUG_NO_COLOR:
+      gst_debug_set_colored (FALSE);
       break;
-    case ARG_MASK_HELP:
-      gst_mask_help ();
+    case ARG_DEBUG_DISABLE:
+      gst_debug_set_active (FALSE);
+      break;
+    case ARG_DEBUG_HELP:
+      gst_debug_help ();
       exit (0);
+#endif
     case ARG_DISABLE_CPU_OPT:
       _gst_enable_cpu_opt = FALSE;
       break;
     case ARG_PLUGIN_SPEW:
       break;
     case ARG_PLUGIN_PATH:
+#ifndef GST_DISABLE_REGISTRY
       split_and_iterate (arg, G_SEARCHPATH_SEPARATOR_S, add_path_func, _user_registry);
+#endif /* GST_DISABLE_REGISTRY */
       break;
     case ARG_PLUGIN_LOAD:
       split_and_iterate (arg, ",", prepare_for_load_plugin_func, NULL);
       break;
+    case ARG_SEGTRAP_DISABLE:
+      _gst_disable_segtrap = TRUE;
+      break;
     case ARG_SCHEDULER:
       gst_scheduler_factory_set_default_name (arg);
       break;
-    case ARG_NOTHREADS:
-      gst_use_threads (FALSE);
-      break;
     case ARG_REGISTRY:
+#ifndef GST_DISABLE_REGISTRY
       g_object_set (G_OBJECT (_user_registry), "location", arg, NULL);
       _gst_registry_fixed = TRUE;
+#endif /* GST_DISABLE_REGISTRY */
       break;
     default:
       g_warning ("option %d not recognized", option->val);
@@ -655,10 +737,17 @@ init_popt_callback (poptContext context, enum poptCallbackReason reason,
  * is turned off, all thread operations such as mutexes and conditionals
  * are turned into NOPs. use this if you want absolute minimal overhead
  * and you don't use any threads in the pipeline.
+ * <note><para>
+ * This function may only be called before threads are initialized. This
+ * usually happens when calling gst_init.
+ * </para></note>
  */
 void
 gst_use_threads (gboolean use_threads)
 {
+  g_return_if_fail (!gst_initialized);
+  g_return_if_fail (g_thread_supported ());
+
   _gst_use_threads = use_threads;
 }
 
@@ -731,4 +820,3 @@ gst_version (guint *major, guint *minor, guint *micro)
   *minor = GST_VERSION_MINOR;
   *micro = GST_VERSION_MICRO;
 }
-
