@@ -25,12 +25,16 @@
 #define __GST_BUFFER_H__
 
 #include <gst/gstdata.h>
+#include <gst/gstclock.h>
 
 G_BEGIN_DECLS
 
 typedef struct _GstBuffer GstBuffer;
 typedef struct _GstBufferPool GstBufferPool;
 
+#define GST_BUFFER_TRACE_NAME		"GstBuffer"
+#define GST_BUFFER_POOL_TRACE_NAME	"GstBufferPool"
+
 extern GType _gst_buffer_type;
 extern GType _gst_buffer_pool_type;
 
@@ -56,10 +60,22 @@ extern GType _gst_buffer_pool_type;
 #define GST_BUFFER_SIZE(buf)			(GST_BUFFER(buf)->size)
 #define GST_BUFFER_MAXSIZE(buf)			(GST_BUFFER(buf)->maxsize)
 #define GST_BUFFER_TIMESTAMP(buf)		(GST_BUFFER(buf)->timestamp)
+#define GST_BUFFER_DURATION(buf)		(GST_BUFFER(buf)->duration)
+#define GST_BUFFER_FORMAT(buf)			(GST_BUFFER(buf)->format)
 #define GST_BUFFER_OFFSET(buf)			(GST_BUFFER(buf)->offset)
+#define GST_BUFFER_OFFSET_END(buf)		(GST_BUFFER(buf)->offset_end)
 #define GST_BUFFER_BUFFERPOOL(buf)		(GST_BUFFER(buf)->pool)
 #define GST_BUFFER_POOL_PRIVATE(buf)		(GST_BUFFER(buf)->pool_private)
 
+#define GST_BUFFER_OFFSET_NONE	((guint64)-1)
+#define GST_BUFFER_MAXSIZE_NONE	((guint)0)
+
+#define GST_BUFFER_DURATION_IS_VALID(buffer)	(GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DURATION (buffer)))
+#define GST_BUFFER_TIMESTAMP_IS_VALID(buffer)	(GST_CLOCK_TIME_IS_VALID (GST_BUFFER_TIMESTAMP (buffer)))
+#define GST_BUFFER_OFFSET_IS_VALID(buffer)	(GST_BUFFER_OFFSET (buffer) != GST_BUFFER_OFFSET_NONE)
+#define GST_BUFFER_OFFSET_END_IS_VALID(buffer)	(GST_BUFFER_OFFSET_END (buffer) != GST_BUFFER_OFFSET_NONE)
+#define GST_BUFFER_MAXSIZE_IS_VALID(buffer)	(GST_BUFFER_MAXSIZE (buffer) != GST_BUFFER_MAXSIZE_NONE)
+
 typedef enum {
   GST_BUFFER_READONLY   = GST_DATA_READONLY,
   GST_BUFFER_SUBBUFFER  = GST_DATA_FLAG_LAST,
@@ -77,22 +93,40 @@ struct _GstBuffer {
   /* pointer to data and its size */
   guint8 		*data;			/* pointer to buffer data */
   guint 		 size;			/* size of buffer data */
-  guint64		 maxsize;		/* max size of this buffer */
+  guint			 maxsize;		/* max size of this buffer */
 
-  guint64		 timestamp;		
+  /* timestamp */
+  GstClockTime		 timestamp;		
+  GstClockTime		 duration;		
+
+  /* media specific offset
+   * for video frames, this could be the number of frames,
+   * for audio data, this could be the number of audio samples,
+   * for file data or compressed data, this could be the number of bytes
+   * offset_end is the last offset contained in the buffer. The format specifies
+   * the meaning of both of them exactly.
+   */
   guint64		 offset;
+  guint64		 offset_end;
 
   /* this is a pointer to the buffer pool (if any) */
   GstBufferPool		*pool;
   /* pointer to pool private data of parent buffer in case of a subbuffer */
   gpointer 		 pool_private;
+
+  gpointer _gst_reserved[GST_PADDING];
 };
 
 /* bufferpools */
 
-typedef GstBuffer*	(*GstBufferPoolBufferNewFunction)	(GstBufferPool *pool, guint64 offset, guint size, gpointer user_data);
-typedef GstBuffer* 	(*GstBufferPoolBufferCopyFunction)	(GstBufferPool *pool, const GstBuffer *buffer, gpointer user_data);
-typedef void	 	(*GstBufferPoolBufferFreeFunction)	(GstBufferPool *pool, GstBuffer *buffer, gpointer user_data);
+typedef GstBuffer*	(*GstBufferPoolBufferNewFunction)	(GstBufferPool *pool, guint64 offset, 
+								 guint size, gpointer user_data);
+typedef GstBuffer* 	(*GstBufferPoolBufferCopyFunction)	(GstBufferPool *pool, 
+								 const GstBuffer *buffer, 
+								 gpointer user_data);
+typedef void	 	(*GstBufferPoolBufferFreeFunction)	(GstBufferPool *pool, 
+								 GstBuffer *buffer, 
+								 gpointer user_data);
 
 struct _GstBufferPool {
   GstData 				data;
@@ -104,24 +138,18 @@ struct _GstBufferPool {
   GstBufferPoolBufferFreeFunction	buffer_free;
 
   gpointer				user_data;
+
+  gpointer _gst_reserved[GST_PADDING];
 };
 
-
-/*< private >*/
-void		_gst_buffer_initialize		(void);
-
-/* functions used by subclasses and bufferpools */
-void		gst_buffer_default_free 	(GstBuffer *buffer);
-GstBuffer*	gst_buffer_default_copy 	(GstBuffer *buffer);
-
-void		gst_buffer_print_stats		(void);
-
 /* allocation */
+GType		gst_buffer_get_type		(void);
 GstBuffer*	gst_buffer_new	 		(void);
 GstBuffer*	gst_buffer_new_and_alloc	(guint size);
 
 /* creating a new buffer from a pool */
-GstBuffer*	gst_buffer_new_from_pool 	(GstBufferPool *pool, guint64 offset, guint size);
+GstBuffer*	gst_buffer_new_from_pool 	(GstBufferPool *pool, 
+						 guint64 offset, guint size);
 
 #define		gst_buffer_set_data(buf, data, size) 	\
 G_STMT_START { 					     	\
@@ -135,7 +163,7 @@ G_STMT_START { 					     	\
 #define		gst_buffer_unref(buf)		gst_data_unref (GST_DATA (buf))
 /* copy buffer */
 #define		gst_buffer_copy(buf)		GST_BUFFER (gst_data_copy (GST_DATA (buf)))
-#define		gst_buffer_needs_copy_on_write(buf)	GST_BUFFER (gst_data_needs_copy_on_write (GST_DATA (buf)))
+#define		gst_buffer_is_writable(buf)	gst_data_is_writable (GST_DATA (buf))
 #define		gst_buffer_copy_on_write(buf)   GST_BUFFER (gst_data_copy_on_write (GST_DATA (buf)))
 #define		gst_buffer_free(buf)		gst_data_free (GST_DATA (buf))
 
@@ -147,8 +175,15 @@ GstBuffer*	gst_buffer_merge		(GstBuffer *buf1, GstBuffer *buf2);
 gboolean	gst_buffer_is_span_fast		(GstBuffer *buf1, GstBuffer *buf2);
 GstBuffer*	gst_buffer_span			(GstBuffer *buf1, guint32 offset, GstBuffer *buf2, guint32 len);
 
+/* --- private --- */
+void		_gst_buffer_initialize		(void);
+
+/* functions used by subclasses and bufferpools */
+void		gst_buffer_default_free 	(GstBuffer *buffer);
+GstBuffer*	gst_buffer_default_copy 	(GstBuffer *buffer);
 
 /* creating a new buffer pools */
+GType		gst_buffer_pool_get_type		(void);
 GstBufferPool*	gst_buffer_pool_new			(GstDataFreeFunction free,
 							 GstDataCopyFunction copy,
 							 GstBufferPoolBufferNewFunction buffer_new,
@@ -159,6 +194,7 @@ GstBufferPool*	gst_buffer_pool_new			(GstDataFreeFunction free,
 /* function used by subclasses and bufferpools */
 void		gst_buffer_pool_default_free		(GstBufferPool *pool);
 
+/* check if pool is usable */
 gboolean	gst_buffer_pool_is_active		(GstBufferPool *pool);
 void		gst_buffer_pool_set_active		(GstBufferPool *pool, gboolean active);
 
@@ -168,7 +204,7 @@ void		gst_buffer_pool_set_active		(GstBufferPool *pool, gboolean active);
 
 /* bufferpool operations */
 #define		gst_buffer_pool_copy(pool)		GST_BUFFER_POOL (gst_data_copy (GST_DATA (pool)))
-#define		gst_buffer_pool_needs_copy_on_write(pool)	GST_BUFFER_POOL (gst_data_needs_copy_on_write (GST_DATA (pool)))
+#define		gst_buffer_pool_is_writable(pool)	GST_BUFFER_POOL (gst_data_is_writable (GST_DATA (pool)))
 #define		gst_buffer_pool_copy_on_write(pool)	GST_BUFFER_POOL (gst_data_copy_on_write (GST_DATA (pool)))
 #define		gst_buffer_pool_free(pool)		gst_data_free (GST_DATA (pool))
 
