@@ -21,24 +21,6 @@ gst_object_flags_get_type (void)
 }
 
 
-/* enumerations from "gstautoplug.h" */
-GType
-gst_autoplug_flags_get_type (void)
-{
-  static GType etype = 0;
-  if (etype == 0) {
-    static const GEnumValue values[] = {
-      { GST_AUTOPLUG_TO_CAPS, "GST_AUTOPLUG_TO_CAPS", "to-caps" },
-      { GST_AUTOPLUG_TO_RENDERER, "GST_AUTOPLUG_TO_RENDERER", "to-renderer" },
-      { GST_AUTOPLUG_FLAG_LAST, "GST_AUTOPLUG_FLAG_LAST", "flag-last" },
-      { 0, NULL, NULL }
-    };
-    etype = g_enum_register_static ("GstAutoplugFlags", values);
-  }
-  return etype;
-}
-
-
 /* enumerations from "gstbin.h" */
 GType
 gst_bin_flags_get_type (void)
@@ -254,6 +236,7 @@ gst_event_type_get_type (void)
       { GST_EVENT_TS_OFFSET, "GST_EVENT_TS_OFFSET", "ts-offset" },
       { GST_EVENT_INTERRUPT, "GST_EVENT_INTERRUPT", "interrupt" },
       { GST_EVENT_NAVIGATION, "GST_EVENT_NAVIGATION", "navigation" },
+      { GST_EVENT_TAG, "GST_EVENT_TAG", "tag" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GstEventType", values);
@@ -685,6 +668,29 @@ gst_scheduler_state_get_type (void)
 }
 
 
+/* enumerations from "gsttag.h" */
+GType
+gst_tag_merge_mode_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_TAG_MERGE_UNDEFINED, "GST_TAG_MERGE_UNDEFINED", "undefined" },
+      { GST_TAG_MERGE_REPLACE_ALL, "GST_TAG_MERGE_REPLACE_ALL", "replace-all" },
+      { GST_TAG_MERGE_REPLACE, "GST_TAG_MERGE_REPLACE", "replace" },
+      { GST_TAG_MERGE_APPEND, "GST_TAG_MERGE_APPEND", "append" },
+      { GST_TAG_MERGE_PREPEND, "GST_TAG_MERGE_PREPEND", "prepend" },
+      { GST_TAG_MERGE_KEEP, "GST_TAG_MERGE_KEEP", "keep" },
+      { GST_TAG_MERGE_KEEP_ALL, "GST_TAG_MERGE_KEEP_ALL", "keep-all" },
+      { GST_TAG_MERGE_COUNT, "GST_TAG_MERGE_COUNT", "count" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstTagMergeMode", values);
+  }
+  return etype;
+}
+
+
 /* enumerations from "gstthread.h" */
 GType
 gst_thread_state_get_type (void)
@@ -793,6 +799,24 @@ gst_result_get_type (void)
 }
 
 
+/* enumerations from "gsturi.h" */
+GType
+gst_uri_type_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GST_URI_UNKNOWN, "GST_URI_UNKNOWN", "unknown" },
+      { GST_URI_SINK, "GST_URI_SINK", "sink" },
+      { GST_URI_SRC, "GST_URI_SRC", "src" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GstURIType", values);
+  }
+  return etype;
+}
+
+
 /* enumerations from "gstregistry.h" */
 GType
 gst_registry_return_get_type (void)
