@@ -53,7 +53,7 @@ typedef enum {
 
 /* we can now override this to be more general in maintainer builds or cvs checkouts */
 #ifndef GST_LEVEL_DEFAULT
-#define GST_LEVEL_DEFAULT GST_LEVEL_ERROR 
+#define GST_LEVEL_DEFAULT GST_LEVEL_NONE
 #endif
 
 /* defines for format (colors etc) - don't change them around, it uses terminal layout 
@@ -162,7 +162,7 @@ void		gst_debug_log			(GstDebugCategory *	category,
 						 GObject *		object,
 						 gchar *		format,
 						 ...)  G_GNUC_PRINTF (7, 8) G_GNUC_NO_INSTRUMENT;
-void		gst_debug_logv			(GstDebugCategory *	category,
+void		gst_debug_log_valist  		(GstDebugCategory *	category,
 						 GstDebugLevel		level,
 						 const gchar *		file,
 						 const gchar *		function,
@@ -180,7 +180,8 @@ void		gst_debug_log_default		(GstDebugCategory *	category,
 						 gchar *		message,
 						 gpointer		unused) G_GNUC_NO_INSTRUMENT;
 
-const gchar *	gst_debug_level_get_name	(GstDebugLevel		level);
+G_CONST_RETURN gchar *
+		gst_debug_level_get_name	(GstDebugLevel		level);
 
 void		gst_debug_add_log_function	(GstLogFunction		func,
 						 gpointer data);
@@ -268,9 +269,11 @@ void		gst_debug_category_set_threshold (GstDebugCategory *	category,
 						 GstDebugLevel		level);
 void		gst_debug_category_reset_threshold (GstDebugCategory *	category);
 GstDebugLevel	gst_debug_category_get_threshold (GstDebugCategory *	category);
-const gchar *	gst_debug_category_get_name	(GstDebugCategory *	category);
+G_CONST_RETURN gchar *
+		gst_debug_category_get_name	(GstDebugCategory *	category);
 guint		gst_debug_category_get_color	(GstDebugCategory *	category);
-const gchar *	gst_debug_category_get_description (GstDebugCategory *	category);
+G_CONST_RETURN gchar *
+		gst_debug_category_get_description (GstDebugCategory *	category);
 GSList *	gst_debug_get_all_categories	(void);
 
 gchar *		gst_debug_construct_term_color	(guint colorinfo);
@@ -350,7 +353,7 @@ extern gboolean			__gst_debug_enabled;
 #endif /* G_HAVE_ISO_VARARGS */
 
 #else /* GST_DEBUG_ENABLE_DEPRECATED */
-/* This is a workaround so the old debugging stuff of Gstreamer 0.6 works.
+/* This is a workaround so the old debugging stuff of GStreamer 0.6 works.
    This is undocumented and will go when 0.8 comes out. */
 
 #ifdef G_HAVE_ISO_VARARGS
@@ -379,7 +382,8 @@ extern gboolean			__gst_debug_enabled;
 /********** function pointer stuff **********/
 void* 		_gst_debug_register_funcptr	(void *			ptr,
 						 gchar *		ptrname);
-const gchar*	_gst_debug_nameof_funcptr 	(void *			ptr);
+G_CONST_RETURN gchar*
+		_gst_debug_nameof_funcptr 	(void *			ptr);
 
 #define GST_DEBUG_FUNCPTR(ptr) (_gst_debug_register_funcptr((void *)(ptr), #ptr) , ptr)
 #define GST_DEBUG_FUNCPTR_NAME(ptr) _gst_debug_nameof_funcptr((void *)ptr)
@@ -388,7 +392,7 @@ const gchar*	_gst_debug_nameof_funcptr 	(void *			ptr);
 
 #ifdef __GNUC__
 #  pragma GCC poison gst_debug_log
-#  pragma GCC poison gst_debug_logv
+#  pragma GCC poison gst_debug_log_valist
 #  pragma GCC poison gst_debug_log_default
 #  pragma GCC poison _gst_debug_category_new
 #endif
