@@ -1,5 +1,6 @@
-#include <gst/gst.h>
 #include <string.h>		/* strerror */
+#include <stdlib.h>		/* strerror */
+#include <gst/gst.h>
 
 #define MAX_THREADS  100
 
@@ -34,7 +35,8 @@ run_test (void *threadid)
 {
   gint i;
   gpointer chunk;
-  sleep(1);
+
+  g_usleep (G_USEC_PER_SEC);
 
   for (i = 0; i<num_allocs; i++) {
     chunk = alloc_chunk ();
@@ -68,7 +70,7 @@ main (gint argc, gchar *argv[])
 
   for(t=0; t < num_threads; t++) {
     error = NULL;
-    threads[t] = g_thread_create (run_test, (void *)t, TRUE, &error);
+    threads[t] = g_thread_create (run_test, GINT_TO_POINTER(t), TRUE, &error);
     if (error) {
       printf ("ERROR: g_thread_create () is %s\n", error->message);
       exit (-1);
