@@ -1,8 +1,7 @@
 #include <gst/gst.h>
 
-#define ITERS 100000
+#define ITERS 100
 #include <stdlib.h>
-#include "mem.h"
 
 static GstElement*
 create_bin (void)
@@ -72,13 +71,17 @@ add_remove_test2 (void)
   g_assert (!GST_OBJECT_FLOATING (element));
   g_assert (!GST_OBJECT_DESTROYED (element));
 
-  gst_object_destroy (GST_OBJECT (element));
+  gst_object_unref (GST_OBJECT (element));
+#if 0
   g_assert (GST_OBJECT_DESTROYED (element));
   gst_object_unref (GST_OBJECT (element));
+#endif
 
   gst_object_unref (GST_OBJECT (bin));
 }
 
+#if 0
+/* This code is bogus */
 static void
 add_remove_test3 (void)
 {
@@ -92,12 +95,15 @@ add_remove_test3 (void)
   gst_bin_add (GST_BIN (bin), element);
   g_assert (!GST_OBJECT_FLOATING (element));
 
-  gst_object_destroy (GST_OBJECT (element));
+  gst_object_unref (GST_OBJECT (element));
   g_assert (gst_bin_get_by_name (GST_BIN (bin), "test1") == NULL);
 
   gst_object_unref (GST_OBJECT (bin));
 }
+#endif
 
+#if 0
+/* This code is bogus */
 static void
 add_remove_test4 (void)
 {
@@ -116,21 +122,24 @@ add_remove_test4 (void)
   gst_bin_add (GST_BIN (bin), bin2);
   g_assert (!GST_OBJECT_FLOATING (bin2));
 
-  gst_object_destroy (GST_OBJECT (bin2));
+  gst_object_unref (GST_OBJECT (bin2));
   g_assert (gst_bin_get_by_name (GST_BIN (bin), "testbin") == NULL);
-  gst_object_destroy (GST_OBJECT (element));
+  gst_object_unref (GST_OBJECT (element));
   g_assert (gst_bin_get_by_name (GST_BIN (bin), "test1") == NULL);
 
   gst_object_unref (GST_OBJECT (bin));
 }
+#endif
 
 int
 main (int argc, gchar *argv[])
 {
   GstElement *bin;
-  long usage1;
+  int usage1;
   gint i, iters;
 
+  gst_alloc_trace_set_flags_all (GST_ALLOC_TRACE_LIVE);
+
   gst_init (&argc, &argv);
 
   if (argc == 2)
@@ -138,18 +147,21 @@ main (int argc, gchar *argv[])
   else
     iters = ITERS;
 
+
   g_print ("starting test\n");
-  usage1 = vmsize();
+
+  usage1 = gst_alloc_trace_live_all ();
+  //gst_alloc_trace_print_all ();
 
   bin = gst_bin_new ("somebin");
   gst_object_unref (GST_OBJECT (bin));
-  g_print ("create/unref new bin %ld\n", vmsize()-usage1);
+  g_print ("create/unref new bin %d\n", gst_alloc_trace_live_all ()-usage1);
 
   for (i=0; i<iters;i++) {
     bin = gst_bin_new ("somebin");
     gst_object_unref (GST_OBJECT (bin));
   }
-  g_print ("create/unref %d bins %ld\n", iters, vmsize()-usage1);
+  g_print ("create/unref %d bins %d\n", iters, gst_alloc_trace_live_all ()-usage1);
 
   bin = gst_bin_new ("somebin");
   g_assert (GST_OBJECT_FLOATING (bin));
@@ -157,7 +169,7 @@ main (int argc, gchar *argv[])
   gst_object_sink (GST_OBJECT (bin));
   g_assert (!GST_OBJECT_FLOATING (bin));
   gst_object_unref (GST_OBJECT (bin));
-  g_print ("create/ref/sink/unref new bin %ld\n", vmsize()-usage1);
+  g_print ("create/ref/sink/unref new bin %d\n", gst_alloc_trace_live_all ()-usage1);
 
 
   for (i=0; i<iters;i++) {
@@ -166,27 +178,31 @@ main (int argc, gchar *argv[])
     gst_object_sink (GST_OBJECT (bin));
     gst_object_unref (GST_OBJECT (bin));
   }
-  g_print ("create/ref/sink/unref %d bins %ld\n", iters, vmsize()-usage1);
+  g_print ("create/ref/sink/unref %d bins %d\n", iters, gst_alloc_trace_live_all ()-usage1);
 
   bin = gst_bin_new ("somebin");
   g_assert (!GST_OBJECT_DESTROYED (bin));
-  gst_object_destroy (GST_OBJECT (bin));
+  gst_object_unref (GST_OBJECT (bin));
+#if 0
   g_assert (GST_OBJECT_DESTROYED (bin));
   gst_object_unref (GST_OBJECT (bin));
-  g_print ("create/destroy/unref new bin %ld\n", vmsize()-usage1);
+#endif
+  g_print ("create/destroy/unref new bin %d\n", gst_alloc_trace_live_all ()-usage1);
   
   for (i=0; i<iters;i++) {
     bin = gst_bin_new ("somebin");
-    gst_object_destroy (GST_OBJECT (bin));
     gst_object_unref (GST_OBJECT (bin));
+#if 0
+    gst_object_unref (GST_OBJECT (bin));
+#endif
   }
-  g_print ("create/destroy/unref %d bin %ld\n", iters, vmsize()-usage1);
+  g_print ("create/destroy/unref %d bin %d\n", iters, gst_alloc_trace_live_all ()-usage1);
 
   bin = gst_bin_new ("somebin");
   gst_object_ref (GST_OBJECT (bin));
   gst_object_unref (GST_OBJECT (bin));
   gst_object_unref (GST_OBJECT (bin));
-  g_print ("create/ref/unref/unref new bin %ld\n", vmsize()-usage1);
+  g_print ("create/ref/unref/unref new bin %d\n", gst_alloc_trace_live_all ()-usage1);
   
   for (i=0; i<iters;i++) {
     bin = gst_bin_new ("somebin");
@@ -194,75 +210,87 @@ main (int argc, gchar *argv[])
     gst_object_unref (GST_OBJECT (bin));
     gst_object_unref (GST_OBJECT (bin));
   }
-  g_print ("create/ref/unref/unref %d bin %ld\n", iters, vmsize()-usage1);
+  g_print ("create/ref/unref/unref %d bin %d\n", iters, gst_alloc_trace_live_all ()-usage1);
 
   bin = gst_bin_new ("somebin");
   gst_object_ref (GST_OBJECT (bin));
-  gst_object_destroy (GST_OBJECT (bin));
   gst_object_unref (GST_OBJECT (bin));
   gst_object_unref (GST_OBJECT (bin));
-  g_print ("craete/ref/destroy/unref/unref new bin %ld\n", vmsize()-usage1);
+#if 0
+  gst_object_unref (GST_OBJECT (bin));
+#endif
+  g_print ("craete/ref/destroy/unref/unref new bin %d\n", gst_alloc_trace_live_all ()-usage1);
   
   for (i=0; i<iters;i++) {
     bin = gst_bin_new ("somebin");
     gst_object_ref (GST_OBJECT (bin));
-    gst_object_destroy (GST_OBJECT (bin));
     gst_object_unref (GST_OBJECT (bin));
     gst_object_unref (GST_OBJECT (bin));
+#if 0
+    gst_object_unref (GST_OBJECT (bin));
+#endif
   }
-  g_print ("craete/ref/destroy/unref/unref %d bins %ld\n", iters, vmsize()-usage1);
+  g_print ("craete/ref/destroy/unref/unref %d bins %d\n", iters, gst_alloc_trace_live_all ()-usage1);
 
   for (i=0; i<iters;i++) {
     bin = gst_bin_new ("somebin");
     gst_object_ref (GST_OBJECT (bin));
     gst_element_set_name (bin, "testing123");
-    gst_object_destroy (GST_OBJECT (bin));
+    gst_object_unref (GST_OBJECT (bin));
     gst_element_set_name (bin, "testing123");
     gst_object_unref (GST_OBJECT (bin));
+#if 0
     gst_object_unref (GST_OBJECT (bin));
+#endif
   }
-  g_print ("craete/ref/destroy/unref/unref %d bins with name %ld\n", iters, vmsize()-usage1);
+  g_print ("craete/ref/destroy/unref/unref %d bins with name %d\n", iters, gst_alloc_trace_live_all ()-usage1);
 
   bin = gst_bin_new ("somebin");
   for (i=0; i<iters;i++) {
     gst_element_set_name (bin, "testing");
   }
   gst_object_unref (GST_OBJECT (bin));
-  g_print ("set name %d times %ld\n", iters, vmsize()-usage1);
+  g_print ("set name %d times %d\n", iters, gst_alloc_trace_live_all ()-usage1);
 
   for (i=0; i<iters;i++) {
     bin = create_bin();
     gst_object_unref (GST_OBJECT (bin));
   }
-  g_print ("create/unref %d bin with children %ld\n", iters, vmsize()-usage1);
+  g_print ("create/unref %d bin with children %d\n", iters, gst_alloc_trace_live_all ()-usage1);
 
   for (i=0; i<iters/2;i++) {
     bin = create_bin_ghostpads();
     gst_object_unref (GST_OBJECT (bin));
   }
-  g_print ("create/unref %d bin with children and ghostpads %ld\n", iters/2, vmsize()-usage1);
+  g_print ("create/unref %d bin with children and ghostpads %d\n", iters/2, gst_alloc_trace_live_all ()-usage1);
 
   for (i=0; i<iters;i++) {
     add_remove_test1();
   }
-  g_print ("add/remove test1 %d in bin %ld\n", iters, vmsize()-usage1);
+  g_print ("add/remove test1 %d in bin %d\n", iters, gst_alloc_trace_live_all ()-usage1);
 
   for (i=0; i<iters;i++) {
     add_remove_test2();
   }
-  g_print ("add/remove test2 %d in bin %ld\n", iters, vmsize()-usage1);
+  g_print ("add/remove test2 %d in bin %d\n", iters, gst_alloc_trace_live_all ()-usage1);
 
+#if 0
   for (i=0; i<iters;i++) {
     add_remove_test3();
   }
-  g_print ("add/destroy/remove test3 %d in bin %ld\n", iters, vmsize()-usage1);
+  g_print ("add/destroy/remove test3 %d in bin %d\n", iters, gst_alloc_trace_live_all ()-usage1);
+#endif
 
+#if 0
   for (i=0; i<iters;i++) {
     add_remove_test4();
   }
-  g_print ("add/destroy/remove test4 %d in bin %ld\n", iters, vmsize()-usage1);
+  g_print ("add/destroy/remove test4 %d in bin %d\n", iters, gst_alloc_trace_live_all ()-usage1);
+#endif
 
-  g_print ("leaked: %ld\n", vmsize()-usage1);
+  g_print ("leaked: %d\n", gst_alloc_trace_live_all ()-usage1);
 
-  return (vmsize()-usage1 ? -1 : 0);
+  //gst_alloc_trace_print_all ();
+
+  return (gst_alloc_trace_live_all ()-usage1 ? -1 : 0);
 }
