@@ -835,15 +835,6 @@ print_element_list (void)
         g_print ("%s:  %s: %s\n", plugin->desc.name,
                 GST_PLUGIN_FEATURE_NAME (factory) ,factory->details.longname);
       }
-#ifndef GST_DISABLE_AUTOPLUG
-      else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
-        GstAutoplugFactory *factory;
-
-        factory = GST_AUTOPLUG_FACTORY (feature);
-        g_print ("%s:  %s: %s\n", plugin->desc.name,
-                GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
-      }
-#endif
 #ifndef GST_DISABLE_INDEX
       else if (GST_IS_INDEX_FACTORY (feature)) {
         GstIndexFactory *factory;
@@ -874,16 +865,6 @@ print_element_list (void)
         g_print ("%s:  %s: %s\n", plugin->desc.name,
                 GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
       }
-#ifndef GST_DISABLE_URI
-      else if (GST_IS_URI_HANDLER (feature)) {
-        GstURIHandler *handler;
-
-        handler = GST_URI_HANDLER (feature);
-        g_print ("%s:  %s: \"%s\" (%s) element \"%s\" property \"%s\"\n",
-                plugin->desc.name, GST_PLUGIN_FEATURE_NAME (handler), handler->uri,
-                handler->longdesc, handler->element, handler->property);
-      }
-#endif
       else {
         g_print ("%s:  %s (%s)\n", plugin->desc.name,
                 GST_PLUGIN_FEATURE_NAME (feature),
@@ -901,7 +882,6 @@ print_plugin_info (GstPlugin *plugin)
   GList *features;
   gint num_features = 0;
   gint num_elements = 0;
-  gint num_autoplug = 0;
   gint num_types = 0;
   gint num_schedulers = 0;
   gint num_indexes = 0;
@@ -913,7 +893,6 @@ print_plugin_info (GstPlugin *plugin)
   g_print ("  Filename:\t%s\n",	    plugin->filename);
   g_print ("  Version:\t%s\n",	    plugin->desc.version);
   g_print ("  License:\t%s\n",	    plugin->desc.license);
-  g_print ("  Copyright:\t%s\n",    plugin->desc.copyright);
   g_print ("  Package:\t%s\n",	    plugin->desc.package);
   g_print ("  Origin URL:\t%s\n",   plugin->desc.origin);
   g_print ("\n");
@@ -933,15 +912,6 @@ print_plugin_info (GstPlugin *plugin)
               factory->details.longname);
       num_elements++;
     }
-#ifndef GST_DISABLE_AUTOPLUG
-    else if (GST_IS_AUTOPLUG_FACTORY (feature)) {
-      GstAutoplugFactory *factory;
-
-      factory = GST_AUTOPLUG_FACTORY (feature);
-      g_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
-      num_autoplug++;
-    }
-#endif
 #ifndef GST_DISABLE_INDEX
     else if (GST_IS_INDEX_FACTORY (feature)) {
       GstIndexFactory *factory;
@@ -985,8 +955,6 @@ print_plugin_info (GstPlugin *plugin)
   g_print ("\n  %d features:\n", num_features);
   if (num_elements > 0)
     g_print ("  +-- %d elements\n", num_elements);
-  if (num_autoplug > 0)
-    g_print ("  +-- %d autopluggers\n", num_autoplug);
   if (num_types > 0)
     g_print ("  +-- %d types\n", num_types);
   if (num_schedulers > 0)
@@ -1055,14 +1023,6 @@ main (int argc, char *argv[])
            g_print ("%s: an index\n", argv[1]);
            return 0;
          }
-#endif
-#ifndef GST_DISABLE_AUTOPLUG
-         feature = gst_registry_pool_find_feature (argv[1],
-                                                   GST_TYPE_AUTOPLUG_FACTORY);
-         if (feature) {
-           g_print ("%s: an autoplugger\n", argv[1]);
-           return 0;
-         }
 #endif
          feature = gst_registry_pool_find_feature (argv[1],
                                                    GST_TYPE_TYPE_FIND_FACTORY);
